@@ -5,7 +5,7 @@ Qt, VTK and matplotlib application to allow for viewing and querying residual st
 0.1 - Inital release
 '''
 __author__ = "M.J. Roy"
-__version__ = "0.1"
+__version__ = "0.2"
 __email__ = "matthew.roy@manchester.ac.uk"
 __status__ = "Experimental"
 __copyright__ = "(c) M. J. Roy, 2021-"
@@ -89,22 +89,34 @@ class main_window(object):
         io_font = QtGui.QFont("Helvetica")
         
         #make display layout
-        display_label = QtWidgets.QLabel("Display")
-        display_label.setFont(head_font)
+        display_box = QtWidgets.QGroupBox('Display')
         #buttons
         self.load_button = QtWidgets.QPushButton('Load')
         self.load_label = QtWidgets.QLabel("Nothing loaded.")
         self.load_label.setWordWrap(True)
         self.load_label.setFont(io_font)
+        self.load_label.setToolTip('Load results file')
         #make combo box for components
         self.component_cb = QtWidgets.QComboBox()
-        self.component_cb.addItems(['S11', 'S22', 'S33'])
+        self.component_cb.setToolTip('Change stress component displayed')
+        # self.component_cb.addItems(['\u03C311', '\u03C322', '\u03C333'])
         self.component_cb.setEnabled(False)
+        self.mesh_display=QtWidgets.QPushButton("Edges off")
+        self.mesh_display.setToolTip('Turn mesh/edges on and off')
+        self.mesh_display.setCheckable(True)
+        self.mesh_display.setChecked(False)
+        self.mesh_display.setEnabled(False)
+        self.extract_boundaries_button = QtWidgets.QPushButton('Extract boundary')
+        self.extract_boundaries_button.setEnabled(False)
+        self.extract_boundaries_button.setToolTip('Extract boundary of model')
+        self.export_STL_button = QtWidgets.QRadioButton("Write STL")
+        self.export_STL_button.setChecked(False)
+        self.export_STL_button.setEnabled(False)
+        
         
         #make contour layout
         contour_layout = QtWidgets.QGridLayout()
-        contour_label = QtWidgets.QLabel("Contours")
-        contour_label.setFont(head_font)
+        contour_box = QtWidgets.QGroupBox('Contours')
         min_contour_label = QtWidgets.QLabel("Min:")
         self.min_contour = QtWidgets.QDoubleSpinBox()
         self.min_contour.setMinimum(-100000)
@@ -115,12 +127,13 @@ class main_window(object):
         self.max_contour.setMaximum(100000)
         num_contour_label = QtWidgets.QLabel("Interval:")
         self.num_contour = QtWidgets.QSpinBox()
+        self.num_contour.setToolTip('Number of entries shown on colorbar')
         self.num_contour.setMinimum(3)
         self.num_contour.setMaximum(20)
         self.num_contour.setValue(5)
         self.update_contours_button = QtWidgets.QPushButton('Update')
+        self.update_contours_button.setToolTip('Update the contour limits and interval')
         self.update_contours_button.setEnabled(False)
-        contour_layout.addWidget(contour_label,0,0,1,6)
         contour_layout.addWidget(min_contour_label,1,0,1,1)
         contour_layout.addWidget(self.min_contour,1,1,1,1)
         contour_layout.addWidget(max_contour_label,1,2,1,1)
@@ -131,98 +144,132 @@ class main_window(object):
         
         
         # line extraction from surface
-        extract_box = QtWidgets.QGridLayout()
-        extract_data_label = QtWidgets.QLabel("Extract")
-        extract_data_label.setFont(head_font)
+        extract_layout = QtWidgets.QGridLayout()
+        extract_box = QtWidgets.QGroupBox('Extract')
+        # labels for axes
+        x_label = QtWidgets.QLabel("x")
+        y_label = QtWidgets.QLabel("y")
+        z_label = QtWidgets.QLabel("z")
         # x, y, z of first point
-        point1_x_label = QtWidgets.QLabel("x0:")
+        start_label = QtWidgets.QLabel("Start")
+        start_label.setToolTip('Start coordinate of line trace')
         self.point1_x_coord = QtWidgets.QDoubleSpinBox()
         self.point1_x_coord.setMinimum(-100000)
         self.point1_x_coord.setMaximum(100000)
-        point1_y_label = QtWidgets.QLabel("y0:")
         self.point1_y_coord = QtWidgets.QDoubleSpinBox()
         self.point1_y_coord.setMinimum(-100000)
         self.point1_y_coord.setMaximum(100000)
-        point1_z_label = QtWidgets.QLabel("z0:")
         self.point1_z_coord = QtWidgets.QDoubleSpinBox()
         self.point1_z_coord.setMinimum(-100000)
         self.point1_z_coord.setMaximum(100000)
 
         # x, y, z of second point
-        point2_x_label = QtWidgets.QLabel("x1:")
+        end_label = QtWidgets.QLabel("End")
+        end_label.setToolTip('End coordinate of line trace')
         self.point2_x_coord = QtWidgets.QDoubleSpinBox()
         self.point2_x_coord.setMinimum(-100000)
         self.point2_x_coord.setMaximum(100000)
-        point2_y_label = QtWidgets.QLabel("y1:")
         self.point2_y_coord = QtWidgets.QDoubleSpinBox()
         self.point2_y_coord.setMinimum(-100000)
         self.point2_y_coord.setMaximum(100000)
-        point2_z_label = QtWidgets.QLabel("z1:")
         self.point2_z_coord = QtWidgets.QDoubleSpinBox()
         self.point2_z_coord.setMinimum(-100000)
         self.point2_z_coord.setMaximum(100000)
         
-        interval_label=QtWidgets.QLabel("Interval:")
+        # x, y, z of clip point
+        clip_label = QtWidgets.QLabel("Clip")
+        clip_label.setToolTip('Tertiary coordinate to specify clipping plane')
+        self.clip_x_coord = QtWidgets.QDoubleSpinBox()
+        self.clip_x_coord.setMinimum(-100000)
+        self.clip_x_coord.setMaximum(100000)
+        self.clip_y_coord = QtWidgets.QDoubleSpinBox()
+        self.clip_y_coord.setMinimum(-100000)
+        self.clip_y_coord.setMaximum(100000)
+        self.clip_z_coord = QtWidgets.QDoubleSpinBox()
+        self.clip_z_coord.setMinimum(-100000)
+        self.clip_z_coord.setMaximum(100000)
+        
+        #clip settings
+        self.clip_active_button=QtWidgets.QPushButton("Update clip")
+        self.clip_active_button.setToolTip('Show/update clipped model')
+        self.clip_active_button.setEnabled(False)
+        
+        interval_label=QtWidgets.QLabel("Line interval:")
         self.extract_interval=QtWidgets.QSpinBox()
+        self.extract_interval.setToolTip('Number of points to extract along line trace')
         self.extract_interval.setValue(50)
         self.extract_interval.setMinimum(3)
         self.extract_interval.setMaximum(1000)
         
-        self.extract_button = QtWidgets.QPushButton('Extract')
+        self.extract_button = QtWidgets.QPushButton('Update line')
+        self.extract_button.setToolTip('Show/update line trace')
         self.extract_button.setEnabled(False)
-        self.export_button = QtWidgets.QPushButton('Export')
-        self.export_button.setEnabled(False)
+        self.export_line_button = QtWidgets.QPushButton('Export line')
+        self.export_line_button.setEnabled(False)
+        self.export_line_button.setToolTip('Export line trace to file')
+
+        
         
         #create figure canvas etc
 
-        #plot
-        #matplotlib finds non-default fonts only with TeX enabled, this is prohibitive from a performance standpoint
-        # rc('font',**{'family':'sans-serif','sans-serif':['helvetica']})
-        # rc('text', usetex=True)
+        #initialize plot
         self.figure = plt.figure(figsize=(4,4))
+        plt.text(0.5, 0.5, "'Update line' for plot", ha='center', style='italic', fontweight = 'bold', color='lightgray', size= 18)
+        plt.axis('off')
         #changes the background of the plot, otherwise white
         # self.figure.patch.set_facecolor((242/255,242/255,242/255))
         self.canvas = FigureCanvas(self.figure)
         self.canvas.setMinimumSize(QtCore.QSize(400, 500))
 
-        #add everything to the extract box
-        extract_box.addWidget(extract_data_label,0,0,1,6)
-        extract_box.addWidget(point1_x_label,1,0,1,1)
-        extract_box.addWidget(self.point1_x_coord,1,1,1,1)
-        extract_box.addWidget(point1_y_label,1,2,1,1)
-        extract_box.addWidget(self.point1_y_coord,1,3,1,1)
-        extract_box.addWidget(point1_z_label,1,4,1,1)
-        extract_box.addWidget(self.point1_z_coord,1,5,1,1)
-        extract_box.addWidget(point2_x_label,2,0,1,1)
-        extract_box.addWidget(self.point2_x_coord,2,1,1,1)
-        extract_box.addWidget(point2_y_label,2,2,1,1)
-        extract_box.addWidget(self.point2_y_coord,2,3,1,1)
-        extract_box.addWidget(point2_z_label,2,4,1,1)
-        extract_box.addWidget(self.point2_z_coord,2,5,1,1)
-        extract_box.addWidget(interval_label,3,0,1,1)
-        extract_box.addWidget(self.extract_interval,3,1,1,1)
-        extract_box.addWidget(self.extract_button,3,2,1,2)
-        extract_box.addWidget(self.export_button,3,4,1,2)
-        extract_box.addWidget(self.canvas,4,0,1,6)
-
-        load_model_box.addWidget(display_label,0,0,1,1)
-        load_model_box.addWidget(self.load_button,0,1,1,1)
-        load_model_box.addWidget(self.component_cb,0,2,1,1)
-        load_model_box.addWidget(self.load_label,1,0,1,3)
-
+        #add everything to the extract layout
+        extract_layout.addWidget(x_label,1,1,1,1)
+        extract_layout.addWidget(y_label,1,2,1,1)
+        extract_layout.addWidget(z_label,1,3,1,1)
+        extract_layout.addWidget(start_label,2,0,1,1)
+        extract_layout.addWidget(self.point1_x_coord,2,1,1,1)
+        extract_layout.addWidget(self.point1_y_coord,2,2,1,1)
+        extract_layout.addWidget(self.point1_z_coord,2,3,1,1)
+        extract_layout.addWidget(end_label,3,0,1,1)
+        extract_layout.addWidget(self.point2_x_coord,3,1,1,1)
+        extract_layout.addWidget(self.point2_y_coord,3,2,1,1)
+        extract_layout.addWidget(self.point2_z_coord,3,3,1,1)
+        extract_layout.addWidget(clip_label,4,0,1,1)
+        extract_layout.addWidget(self.clip_x_coord,4,1,1,1)
+        extract_layout.addWidget(self.clip_y_coord,4,2,1,1)
+        extract_layout.addWidget(self.clip_z_coord,4,3,1,1)
+        extract_layout.addWidget(self.extract_button,5,2,1,1)
+        extract_layout.addWidget(self.clip_active_button,5,3,1,1)
+        extract_layout.addWidget(self.canvas,6,0,1,4)
         
-        frame_style = QtWidgets.QFrame.HLine | QtWidgets.QFrame.Sunken
+
+        load_model_box.addWidget(self.load_button,0,0,1,1)
+        load_model_box.addWidget(self.component_cb,0,1,1,1)
+        load_model_box.addWidget(self.mesh_display,0,2,1,1)
+        load_model_box.addWidget(self.load_label,1,0,1,3)
+        load_model_box.addWidget(self.extract_boundaries_button,2,1,1,1)
+        load_model_box.addWidget(self.export_STL_button,2,2,1,1)
+        
+        #add layouts to boxes
+        display_box.setLayout(load_model_box)
+        contour_box.setLayout(contour_layout)
+        evlayout=QtWidgets.QVBoxLayout()
+        evbutton_layout = QtWidgets.QHBoxLayout()
+        evbutton_layout.addWidget(interval_label)
+        evbutton_layout.addWidget(self.extract_interval)
+        verticalSpacer = QtWidgets.QSpacerItem(200, 20, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
+        evbutton_layout.addItem(verticalSpacer)
+        evbutton_layout.addWidget(self.export_line_button)
+        
+        evlayout.addLayout(extract_layout)
+        evlayout.addLayout(evbutton_layout)
+        
+        extract_box.setLayout(evlayout)
         
         lvlayout=QtWidgets.QVBoxLayout()
-        lvlayout.addLayout(load_model_box)
-        horiz_line1 = QtWidgets.QFrame()
-        horiz_line1.setFrameStyle(frame_style)
-        lvlayout.addWidget(horiz_line1)
-        lvlayout.addLayout(contour_layout)
-        horiz_line2 = QtWidgets.QFrame()
-        horiz_line2.setFrameStyle(frame_style)
-        lvlayout.addWidget(horiz_line2)
-        lvlayout.addLayout(extract_box)
+        lvlayout.addWidget(display_box)
+        lvlayout.addWidget(contour_box)
+        lvlayout.addWidget(extract_box)
+
         lvlayout.addStretch(1)
         
         mainlayout.addWidget(self.vtkWidget)
@@ -256,20 +303,59 @@ class interactor(QtWidgets.QWidget):
         make_logo(self.ren)
         
         self.ui.load_button.clicked.connect(self.load_vtu)
-        self.ui.component_cb.currentIndexChanged.connect(self.reset_model)
+        self.ui.mesh_display.clicked.connect(self.toggle_edges)
+        self.ui.clip_active_button.clicked.connect(self.clip)
         self.ui.extract_button.clicked.connect(self.extract)
-        self.ui.export_button.clicked.connect(self.export)
-        self.ui.update_contours_button.clicked.connect(self.draw_model)
+        self.ui.export_line_button.clicked.connect(self.export_line)
+        self.ui.update_contours_button.clicked.connect(self.update_scale_bar)
+        self.ui.extract_boundaries_button.clicked.connect(self.get_boundaries)
     
-    def contextMenuEvent(self, event):
-        contextMenu = QtWidgets.QMenu(self)
-        newAct = contextMenu.addAction("New")
-        openAct = contextMenu.addAction("Open")
-        quitAct = contextMenu.addAction("Quit")
-        action = contextMenu.exec_(self.mapToGlobal(event.pos()))
-        if action == quitAct:
-            self.close()
+    def get_boundaries(self):
+        '''
+        Extracts the boundaries of the model to a polydata object, if export_STL is selected then ask for where to save it.
+        '''
+        #can't be activated unless load_model has run
+        extract_surface = vtk.vtkDataSetSurfaceFilter()
+        extract_surface.SetInputDataObject(self.vtu_output)
+        extract_surface.Update()
+        self.model_boundary = vtk.vtkPolyData()
+        self.model_boundary = extract_surface.GetOutput()
+        msg = 'Extracted boundaries.'
+        
+        if self.ui.export_STL_button.isChecked():
+            #get file name
+            fileo, _ = get_save_file('*.stl')
+            if fileo is None:
+                return
+            writer = vtk.vtkSTLWriter()
+            writer.SetFileName(fileo)
+            writer.SetInputData(self.model_boundary)
+            msg = 'Extracted boundaries and wrote STL file.' #overwrite msg if stl was written
+            writer.Write()
+        
+        self.display_info(msg)
+        self.ui.vtkWidget.update()
+
     
+    def toggle_edges(self):
+        '''
+        changes the visibility of edges on the active mesh_actor
+        '''
+        if hasattr(self,'mesh_actor'):
+            if self.ui.mesh_display.isChecked():
+                self.mesh_actor.GetProperty().EdgeVisibilityOff()
+            else:
+                self.mesh_actor.GetProperty().EdgeVisibilityOn()
+        
+        if hasattr(self,'clipped_actor'):
+            if self.ui.mesh_display.isChecked():
+                self.clipped_actor.GetProperty().EdgeVisibilityOff()
+            else:
+                self.clipped_actor.GetProperty().EdgeVisibilityOn()
+        
+        self.ui.vtkWidget.update()
+        
+
     def write_h5(self):
         '''
         method which writes to an hdf5 file if there is anything to write
@@ -286,61 +372,76 @@ class interactor(QtWidgets.QWidget):
             w.SetInputConnection(reader.GetOutputPort())
             w.SetFileName(self.file)
             w.Update()
-            self.info_actor = generate_info_actor(self.ren,'Saved to data file.')
+            self.display_info('Saved to data file.')
             with h5py.File(self.file, 'r+') as f:
                 f.attrs['date_modified'] = datetime.today().strftime('%Y-%m-%d %H:%M:%S')
+                #if there's an stl file
+                if hasattr(self,'model_boundary'):
+                    # get points and verts from the boundary polydata, clearing anything that might already be there
+                    del f['model_boundary/points']
+                    del f['model_boundary/vertices']
+                    np_pts = v2n(self.model_boundary.GetPoints().GetData())
+                    np_verts = v2n(self.model_boundary.GetPolys().GetData())
+                    f.create_dataset('model_boundary/points', data=np_pts)
+                    f.create_dataset('model_boundary/vertices', data=np_verts)
+                self.display_info('Saved to data file.')
         else:
             return
 
-    def reset_model(self):
-        '''
-        Clears variable which triggers an update redrawing the model with the relevant component ranges.
-        '''
-        del self.vtu_output
-        self.draw_model()
-
     def draw_model(self):
         '''
         Main method which updates the display of the model.
         '''
-        self.component = self.ui.component_cb.currentText()
+        
+        #Logic which bypasses the initial call from the combobox on loading
+        if self.ui.component_cb.currentText() != '':
+            self.component = self.ui.component_cb.currentText()
+        
+        
+        if self.ui.mesh_display.isChecked():
+            edges = False
+        else:
+            edges = True
         #clear all actors
         self.ren.RemoveAllViewProps()
         if hasattr(self,'active_vtu') and not hasattr(self,'vtu_output'):
-            mesh_actor, self.vtu_output, self.mesh_lut, r = read_model_data(\
-            self.active_vtu, \
-            self.component, \
-            None, None)
-            #update contour limits
-            self.ui.min_contour.setValue(r[0])
-            self.ui.max_contour.setValue(r[1])
+            #read the model data
+            self.vtu_output, components = read_model_data(self.active_vtu)
+            #update the combobox with the components
+            self.ui.component_cb.addItems(components)
+            self.component = self.ui.component_cb.currentText()
+
+            self.mesh_actor, self.mesh_mapper, self.mesh_lut, range = generate_ug_actor(self.vtu_output, self.component, edges)
+
+            self.ui.extract_boundaries_button.setEnabled(True)
+            self.ui.export_STL_button.setEnabled(True)
             self.ui.update_contours_button.setEnabled(True)
             self.ui.extract_button.setEnabled(True)
-            self.ui.export_button.setEnabled(True)
-        elif hasattr(self,'vtu_output'):
-            mesh_actor, self.vtu_output, _, _ = read_model_data(\
-            self.active_vtu, \
-            self.component, \
-            self.mesh_lut, \
-            (self.ui.min_contour.value(),self.ui.max_contour.value()))
-            print(r)
+            self.ui.export_line_button.setEnabled(True)
+        elif hasattr(self,'vtu_output'): #then operate on the existing vtu's contents
+            self.mesh_actor, self.mesh_mapper, self.mesh_lut, range = generate_ug_actor(self.vtu_output, self.component, edges)
         else:
             return
         
+        #update contour limits
+        self.ui.min_contour.setValue(range[0])
+        self.ui.max_contour.setValue(range[1])
+        
+        
         #create scale bar
         scalar_bar_widget = vtk.vtkScalarBarWidget()
         scalarBarRep = scalar_bar_widget.GetRepresentation()
         scalarBarRep.GetPositionCoordinate().SetValue(0.01,0.01)
         scalarBarRep.GetPosition2Coordinate().SetValue(0.09,0.9)
-        sb_actor=scalar_bar_widget.GetScalarBarActor()
+        self.sb_actor=scalar_bar_widget.GetScalarBarActor()
 
-        sb_actor.SetNumberOfLabels(self.ui.num_contour.value())
+        self.sb_actor.SetNumberOfLabels(self.ui.num_contour.value())
 
-        sb_actor.SetLookupTable(self.mesh_lut)
-        sb_actor.SetTitle('MPa')
+        self.sb_actor.SetLookupTable(self.mesh_lut)
+        self.sb_actor.SetTitle('MPa')
 
 
-        #attempt to change scalebar properties [ineffective]
+        #attempt to change scalebar properties
         propT = vtk.vtkTextProperty()
         propL = vtk.vtkTextProperty()
         propT.SetColor(0,0,0)
@@ -351,14 +452,14 @@ class interactor(QtWidgets.QWidget):
         propL.BoldOff()
         propL.SetFontSize(1)
         propT.SetFontSize(1)
-        sb_actor.GetLabelTextProperty().SetColor(0,0,0)
-        sb_actor.GetTitleTextProperty().SetColor(0,0,0)
-        sb_actor.GetLabelTextProperty().SetFontSize(1)
-        sb_actor.GetTitleTextProperty().SetFontSize(1)
-        sb_actor.SetLabelFormat("%.1f")
+        self.sb_actor.GetLabelTextProperty().SetColor(0,0,0)
+        self.sb_actor.GetTitleTextProperty().SetColor(0,0,0)
+        self.sb_actor.GetLabelTextProperty().SetFontSize(1)
+        self.sb_actor.GetTitleTextProperty().SetFontSize(1)
+        self.sb_actor.SetLabelFormat("%.1f")
 
-        self.ren.AddActor(mesh_actor)
-        self.ren.AddActor(sb_actor)
+        self.ren.AddActor(self.mesh_actor)
+        self.ren.AddActor(self.sb_actor)
         self.axis_actor = generate_axis_actor(self.vtu_output,self.ren)
         self.ren.AddActor(self.axis_actor)
         
@@ -367,7 +468,17 @@ class interactor(QtWidgets.QWidget):
         self.ren.ResetCamera()
         
         self.ui.vtkWidget.update()
-        QtWidgets.QApplication.processEvents()
+    
+    def update_scale_bar(self):
+        '''
+        updates the active scale bar with limits and number of intervals from ui
+        '''
+        r = (self.ui.min_contour.value(),self.ui.max_contour.value())
+        self.mesh_mapper.SetScalarRange(r[0], r[1])
+        if hasattr(self,'clip_mapper'):
+            self.clip_mapper.SetScalarRange(r[0], r[1])
+        self.sb_actor.SetNumberOfLabels(self.ui.num_contour.value())
+        self.ui.vtkWidget.update()
 
     def extract(self):
         '''
@@ -379,11 +490,12 @@ class interactor(QtWidgets.QWidget):
         p1 = [self.ui.point1_x_coord.value(), self.ui.point1_y_coord.value(), self.ui.point1_z_coord.value()]
         p2 = [self.ui.point2_x_coord.value(), self.ui.point2_y_coord.value(), self.ui.point2_z_coord.value()]
         self.q = line_query(self.vtu_output,p1,p2,self.ui.extract_interval.value(),self.component)
-        self.x = range(len(self.q))
+        self.x = range(self.q.shape[0])
         self.ui.figure.clear()
-        QtWidgets.QApplication.processEvents()
+        
+        # print(self.x,self.q)
         ax = self.ui.figure.add_subplot(111)
-        ax.scatter(self.x,self.q)
+        ax.scatter(self.x,self.q[:,-1])
         ax.set_ylabel("%s (MPa)"%self.component)
         ax.set_xlabel("Point number")
         ax.grid(b=True, which='major', color='#666666', linestyle='-')
@@ -425,11 +537,61 @@ class interactor(QtWidgets.QWidget):
         colors = vtk.vtkNamedColors()
         self.line_actor.GetProperty().SetColor(colors.GetColor3d("Violet"))
         self.ren.AddActor(self.line_actor)
-        self.ui.export_button.setEnabled(True)
+        self.ui.export_line_button.setEnabled(True)
         
         self.ui.vtkWidget.update()
+
+    def clip(self):
+        '''
+        Activates clipping by hiding mesh_actor and replacing it with a clipped actor based on the points set in the text box. Clipping plane is specified by the plane defined by 'Start','End' and 'Clip'.
+        Clipping is removed by specifying zeros for the third point, and by virtue avoids a divide by zero error when calculating the clipping plane normal.
+        '''
         
-    def export(self):
+        if hasattr(self,'clipped_actor'):
+            self.ren.RemoveActor(self.clipped_actor)
+            
+        #read points for plane
+        p1 = np.array([self.ui.point1_x_coord.value(), self.ui.point1_y_coord.value(), self.ui.point1_z_coord.value()])
+        p2 = np.array([self.ui.point2_x_coord.value(), self.ui.point2_y_coord.value(), self.ui.point2_z_coord.value()])
+        p3 = np.array([self.ui.clip_x_coord.value(), self.ui.clip_y_coord.value(), self.ui.clip_z_coord.value()])
+        
+        c = p3 == np.zeros(3)
+        if c.all() and self.mesh_actor.GetVisibility() == 0: #no clipping plane (p3 = 0,0,0) is specified & mesh is hidden
+            flip_visible(self.mesh_actor)
+
+        elif not c.all():
+            clipPlane = vtk.vtkPlane()
+            clipPlane.SetOrigin(((p1+p2)/2).tolist())
+            #solve cross product between p1,p2 and p2,p3
+            xnorm = np.cross((p2-p1),(p3-p2))
+            xnorm = xnorm / np.sqrt(np.sum(xnorm**2))
+            clipPlane.SetNormal(xnorm.tolist())
+            
+            clipper = vtk.vtkTableBasedClipDataSet() #needs to be table based, otherwise the grid is interpolated
+            clipper.SetClipFunction(clipPlane)
+            clipper.SetInputData(self.vtu_output) #needs to remain vtk object
+            clipper.GenerateClippedOutputOn()
+            clipper.Update()
+
+            self.clip_mapper = vtk.vtkDataSetMapper()
+            self.clip_mapper.SetInputData(clipper.GetClippedOutput())
+            self.clip_mapper.SetLookupTable(self.mesh_lut)
+            self.clip_mapper.SetScalarRange(self.vtu_output.GetScalarRange())
+
+            self.clipped_actor = vtk.vtkActor()
+            self.clipped_actor.SetMapper(self.clip_mapper)
+            if self.ui.mesh_display.isChecked():
+                self.clipped_actor.GetProperty().EdgeVisibilityOff()
+            else:
+                self.clipped_actor.GetProperty().EdgeVisibilityOn()
+            if self.mesh_actor.GetVisibility() == 1:
+                flip_visible(self.mesh_actor)
+            self.ren.AddActor(self.clipped_actor)
+        
+        self.ui.vtkWidget.update()
+        # QtWidgets.QApplication.processEvents()
+        
+    def export_line(self):
         """
         Collects data from ui, writes to a valid file
         """
@@ -441,7 +603,9 @@ class interactor(QtWidgets.QWidget):
         np.savetxt(fileo,
         np.column_stack((self.x,self.q)), 
         delimiter=',',
-        header = "%s\nPoint number, %s (MPa)"%(self.active_vtu,self.component))
+        header = "%s\nPoint number, x, y, z, %s (MPa)"%(self.active_vtu,self.component),
+        fmt='%i, %.3f, %.3f, %.3f, %.3f')
+        self.display_info('Line exported.')
 
     def load_vtu(self):
         """
@@ -452,39 +616,41 @@ class interactor(QtWidgets.QWidget):
         if filep is None:
             return
         if not(os.path.isfile(filep)):
-            print('Data file invalid.')
+            self.display_info('Invalid *.vtu file.')
             return
         
         self.active_vtu = filep
         self.ui.load_label.setText(filep)
         #call draw_model
         self.ui.component_cb.setEnabled(True)
+        self.ui.mesh_display.setEnabled(True)
+        self.ui.clip_active_button.setEnabled(True)
         self.draw_model()
-        self.info_actor = generate_info_actor(self.ren,'Loaded model from VTU file.')
+        #connect after the first time draw_model runs and finds all components
+        self.ui.component_cb.currentIndexChanged.connect(self.draw_model)
+        
             
     def load_h5(self):
-        
         if self.file is None:
             self.file, _ = get_file("*.OpenRS")
         
         if self.file is not None:
             #check the file has a populated model object
             with h5py.File(self.file, 'r') as f:
-                if "model/piece0" not in f:
-                    self.info_actor = generate_info_actor(self.ren,'Model data could not be loaded.')
-                    return
-        
-        #otherwise read it
-        r = HDF5vtkug_reader()
-        r.SetFileName(self.file)
-        r.Update()
-        self.active_vtu = r.GetOutputDataObject(0).GetBlock(0)
-        self.ui.load_label.setText(self.file)
-        #call draw_model
-        self.ui.component_cb.setEnabled(True)
-        self.draw_model()
-        self.info_actor = generate_info_actor(self.ren,'Loaded model from data file.')
-                
+                if "model_data/piece0" not in f:
+                    self.display_info('Model data could not be loaded.')
+                    
+            r = HDF5vtkug_reader()
+            r.SetFileName(self.file)
+            r.Update()
+            self.active_vtu = r.GetOutputDataObject(0).GetBlock(0)
+            self.ui.load_label.setText(self.file)
+            #call draw_model
+            self.ui.component_cb.setEnabled(True)
+            self.ui.mesh_display.setEnabled(True)
+            self.draw_model()
+            self.display_info('Loaded model from data file.')
+        self.ui.vtkWidget.update() #for display of info_actor
 
     def keypress(self, obj, event):
         '''
@@ -520,7 +686,7 @@ class interactor(QtWidgets.QWidget):
             writer.SetFileName("OpenRS_capture.png")
             writer.Write()
             self.ren.SetBackground(colors.GetColor3d("aliceblue"))
-            self.info_actor = generate_info_actor(self.ren,'Image saved.')
+            self.display_info('Image saved.')
         
         elif key=="r":
             flip_visible(self.axis_actor)
@@ -534,10 +700,20 @@ class interactor(QtWidgets.QWidget):
         else:
             pass
 
-def read_model_data(vtu, component, lut, range):
+    def display_info(self,msg):
+        '''
+        Checks if there's an info_actor and removes it before displaying another one
+        '''
+        if hasattr(self,'info_actor'):
+            self.ren.RemoveActor(self.info_actor)
+        self.info_actor = generate_info_actor(msg,self.ren)
+        self.ren.AddActor(self.info_actor)
+            
+def read_model_data(vtu):
     '''
-    Read an unstructured grid from an XML formated vtu file, or operate on a ug object, setting the output to be a 'component'. If not specified, generate a lookup table and range based on the specified component, otherwise, use the lookup table specified with the given range.
+    Read an unstructured grid from an XML formated vtu file, or operate on a ug object, returning the ug and component names.
     '''
+
     #If read is true, then vtuname is a VTU file
     if type(vtu) is str:
         reader = vtk.vtkXMLUnstructuredGridReader()
@@ -546,31 +722,40 @@ def read_model_data(vtu, component, lut, range):
         output = reader.GetOutput()
     else: #coming from hdf5reader
         output = vtu
-        
-    output.GetPointData().SetActiveScalars(component)
-    
-    if lut is None or range is None:
-        #build lookup table according to field
-        lut = vtk.vtkLookupTable()
-        lut.SetHueRange(0.667, 0)
-        lut.Build()
-        range = output.GetScalarRange()
-    
+    #get the component names
+    components = []
+    for index in range(output.GetPointData().GetNumberOfArrays()):
+        components.append(output.GetPointData().GetArrayName(index))
+
+    return output, components
+
+def generate_ug_actor(ug, component, edges):
+    '''
+    Return an actor with a look up table and range of component selected. Edges are on if true.
+    '''
+    ug.GetPointData().SetActiveScalars(component)
+
+    #build lookup table according to field
+    lut = vtk.vtkLookupTable()
+    lut.SetHueRange(0.667, 0)
+    lut.Build()
+    range = ug.GetScalarRange()
 
     # map data set
     mesh_mapper = vtk.vtkDataSetMapper()
-    mesh_mapper.SetInputData(output)
+    mesh_mapper.SetInputData(ug)
     mesh_mapper.SetScalarRange(range)
     mesh_mapper.SetLookupTable(lut)
 
-
-    # Create the Actor
     actor = vtk.vtkActor()
     actor.SetMapper(mesh_mapper)
-    actor.GetProperty().EdgeVisibilityOn()
+    if edges:
+        actor.GetProperty().EdgeVisibilityOn()
+    else:
+        actor.GetProperty().EdgeVisibilityOff()
     actor.GetProperty().SetLineWidth(0)
 
-    return actor, output, lut, range
+    return actor, mesh_mapper, lut, range
 
 if __name__ == "__main__":
     if len(sys.argv)>1:
